package labss.lang.il.model;

/**
 * Fornisce l'intefaccia per creare tutti gli oggetti che rappresentano le entit del linguaggio.<p/>
 * Ogni gruppo  tenuto a scrivere una classe chiamata <tt>ALanguageFactory</tt> che implementi questa interfaccia.
 *
 *
 * @author Riccardo Solmi
 * @version 1.0
 * @see Statement
 * @see Expression
 */

public interface LanguageFactory {
  // istruzioni

  /**
   * Costruisce un oggetto che rappresenta una istruzione <tt>while</tt>. <p/>
   *
   * L'istruzione <tt>while</tt> si comporta come l'analoga Java.<br/>
   * Finch l'espressione  vera ripete l'esecuzione dell'istruzione.<p/>
   *
   * L'esecuzione di una istruzione <tt>while</tt> che contiene una espressione
   * non booleana deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>.
   *
   * @param exp Espressione da valutare.
   * @param stm Istruzione da ripetere.
   * @return Un oggetto che rappresenta una istruzione <tt>while</tt>.
   */
  public Statement createWhile(Expression exp, Statement stm);

  /**
   * Costruisce un oggetto che rappresenta una istruzione <tt>if</tt>. <p/>
   * 
   * L'istruzione <tt>if</tt> si comporta come l'analoga Java.<br/>
   * Se l'espressione  vera viene eseguita l'istruzione.<p/>
   *
   * L'esecuzione di una istruzione <tt>if</tt> che contiene una espressione
   * non booleana deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>.
   *
   * @param exp Espressione da valutare.
   * @param stm Istruzione da eseguire se exp  vera.
   * @return Un oggetto che rappresenta una istruzione <tt>if</tt>.
   */
  public Statement createIf(Expression exp, Statement stm);

  /**
   * Costruisce un oggetto che rappresenta una istruzione <tt>if</tt>. <p/>
   *
   * L'istruzione <tt>if</tt> si comporta come l'analoga Java.<br/>
   * Se l'espressione  vera viene eseguito il ramo <i>then</i> altrimenti quello <i>else</i>.<p/>
   *
   * L'esecuzione di una istruzione <tt>if</tt> che contiene una espressione
   * non booleana deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>.
   *
   * @param exp Espressione da valutare.
   * @param thenStm Istruzione da eseguire se exp  vera.
   * @param elseStm Istruzione da eseguire se exp  falsa. 
   * @return Un oggetto che rappresenta una istruzione <tt>if</tt>.
   */
  public Statement createIf(Expression exp, Statement thenStm, Statement elseStm);

  /**
   * Costruisce un oggetto che rappresenta una istruzione di assegnamento. <p/>
   *
   * L'istruzione di assegnamento si comporta come l'analoga Java.<br/>
   * L'espressione viene valutata e il suo valore assegnato alla variabile indicata.<p/>
   *
   * L'esecuzione di una istruzione di assegnamento su una <tt>var</tt> che non rappresenta
   * una variabile deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>.
   *
   * @param var Variabile da modificare.
   * @param exp Espressione da valutare. 
   * @return Un oggetto che rappresenta una istruzione di assegnamento.
   * @see #createVariable(String)
   */
  public Statement createAssignment(Expression var, Expression exp);

  /**
   * Costruisce un oggetto che rappresenta una sequenza di istruzioni. <p/>
   *
   * L'istruzione blocco si comporta come l'analoga Java.<br/>
   * Vengono eseguite in sequenza e nell'ordine passato tutte le istruzioni.<p/>
   *
   * @param stm Array di istruzioni.
   * @return Un oggetto che rappresenta una sequenza di istruzioni.
   */
  public Statement createBlock(Statement[] stm);


  //espressioni

  /**
   * Costruisce un oggetto che rappresenta una costante booleana. <p/>
   *
   * @param val Il valore della costante booleana.
   * @return Un oggetto che rappresenta una costante booleana.
   */
  public Literal createLiteral(boolean val);

  /**
   * Costruisce un oggetto che rappresenta una costante intera. <p/>
   *
   * @param val Il valore della costante intera.
   * @return Un oggetto che rappresenta una costante intera.
   */
  public Literal createLiteral(int val);

  /**
   * Costruisce un oggetto che rappresenta una variabile. <p/>
   *
   * Una variabile non ha tipo e pu contenere un valore costante.<br/>
   * Ad una variabile pu essere assegnato un valore usando una istruzione
   * di assegnamento. <p/>
   *
   * La valutazione di una variabile non inizializzata deve provocare una
   * eccezione di tipo <tt>IllegalStateException</tt>. <p/>
   *
   * @param name Il nome della variabile.
   * @return Un oggetto che rappresenta una variabile.
   * @see #createAssignment(Expression, Expression)
   */
  public Expression createVariable(String name);

  /**
   * Costruisce un oggetto che rappresenta una espressione somma. <p/>
   *
   * La valutazione di una espressione somma su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione somma  di tipo intero.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione somma.
   */
  public Expression createAddition(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione sottrazione. <p/>
   *
   * La valutazione di una espressione sottrazione su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione sottrazione  di tipo intero.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione sottrazione.
   */
  public Expression createSubtraction(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione moltiplicazione. <p/>
   *
   * La valutazione di una espressione moltiplicazione su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione moltiplicazione  di tipo intero.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione moltiplicazione.
   */
  public Expression createMultiplication(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione divisione. <p/>
   *
   * La valutazione di una espressione divisione su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione divisione  di tipo intero.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione divisione.
   */
  public Expression createDivision(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione <tt>and</tt>. <p/>
   *
   * La valutazione di una espressione <tt>and</tt> su operandi che non siano entrambi
   * booleani deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione <tt>and</tt>  di tipo booleano.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione <tt>and</tt>.
   */
  public Expression createAnd(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione <tt>or</tt>. <p/>
   *
   * La valutazione di una espressione <tt>or</tt> su operandi che non siano entrambi
   * booleani deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione <tt>or</tt>  di tipo booleano.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione <tt>or</tt>.
   */
  public Expression createOr(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione <tt>not</tt>. <p/>
   *
   * La valutazione di una espressione <tt>not</tt> su un operando che non sia
   * booleano deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione <tt>not</tt>  di tipo booleano.
   *
   * @param exp Operando.
   * @return Un oggetto che rappresenta una espressione <tt>not</tt>.
   */
  public Expression createNot(Expression exp);

  /**
   * Costruisce un oggetto che rappresenta una espressione <i>uguale a</i>. <p/>
   *
   * La valutazione di una espressione <i>uguale a</i> su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione <i>uguale a</i>  di tipo booleano.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione <i>uguale a</i>.
   */
  public Expression createEquals(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione <i>minore di</i>. <p/>
   *
   * La valutazione di una espressione <i>minore di</i> su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione <i>minore di</i>  di tipo booleano.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione <i>minore di</i>.
   */
  public Expression createLessThan(Expression exp1, Expression exp2);

  /**
   * Costruisce un oggetto che rappresenta una espressione <i>maggiore di</i>. <p/>
   *
   * La valutazione di una espressione <i>maggiore di</i> su operandi che non siano entrambi
   * interi deve provocare una eccezione di tipo <tt>IllegalArgumentException</tt>. <p/>
   *
   * Il risultato della valutazione di una espressione <i>maggiore di</i>  di tipo booleano.
   *
   * @param exp1 Primo operando.
   * @param exp2 Secondo operando.
   * @return Un oggetto che rappresenta una espressione <i>maggiore di</i>.
   */
  public Expression createGreaterThan(Expression exp1, Expression exp2);
}
